"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const fs_extra_1 = __importDefault(require("fs-extra"));
const http_1 = __importDefault(require("http"));
const path_1 = __importDefault(require("path"));
const tar_1 = __importDefault(require("tar"));
const cwd = process.cwd();
function downloadFile(url, targetPath) {
    return new Promise((resolve) => {
        const target = fs_extra_1.default.createWriteStream(targetPath);
        http_1.default.get(url, (response) => {
            response
                .pipe(target)
                .on('close', () => {
                resolve(true);
            })
                .on('finish', () => {
                resolve(true);
            })
                .on('error', () => {
                resolve(false);
            });
        });
    });
}
async function downloadFederationTypes(remotes, outputDir, remoteFileName = '[name]-dts.tgz') {
    const savedDts = [];
    for (const [, value] of Object.entries(remotes)) {
        const [moduleName, entryFileURL] = value.split('@');
        const outputPath = outputDir.replace('[name]', moduleName);
        const dtsFileName = remoteFileName.replace('[name]', moduleName);
        const dtsFileURL = new url_1.URL(dtsFileName, entryFileURL).href;
        const targetFile = path_1.default.resolve(outputPath, dtsFileName);
        await fs_extra_1.default.ensureDir(outputPath);
        console.log('Downloading: ', dtsFileURL);
        const saved = await downloadFile(dtsFileURL, targetFile);
        if (saved) {
            savedDts.push(targetFile);
            console.log('Saved: ', targetFile);
            await tar_1.default.x({
                file: targetFile,
                cwd: outputPath,
            });
            await fs_extra_1.default.remove(targetFile);
        }
        else {
            console.error('Failed to download remote DTS: ', dtsFileURL);
        }
    }
}
class WebpackRemoteTypesPlugin {
    constructor(options) {
        this.options = options;
    }
    apply(compiler) {
        compiler.hooks.beforeCompile.tapPromise('WebpackRemoteTypesPlugin', () => {
            return downloadFederationTypes(this.options.remotes, path_1.default.resolve(cwd, this.options.outputDir), this.options.remoteFileName);
        });
    }
}
exports.default = WebpackRemoteTypesPlugin;
